# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

# THIS FILE IS GENERATED FROM SETUP.PY
version = '0.6.0'
full_version = '0.6.0'
git_revision = '8a6dfcff899abf9236edd4a000579a0c5a5552d2'
is_released = True

if not is_released:
    version = full_version
